//=============================================================================
// PropBreakableMP.
//=============================================================================
class PropBreakableMP extends PropBreakable
	hidecategories(PropBreakable);
// Rewriting a bunch of stuff here, don't allow mapmakers to change the values from PropBreakable parent, cuz they won't work.

var() float MinDamage; // Minimum amount of damage required to "make a dent", so to say
var() float HealthMax; // How much health I have before exploding

var() float RespawnTime; // How long in seconds inbetween respawns. If 0, respawns disabled.
var() class<Emitter> RespawnEffect; // Emitter class for the respawning effect
var() class<Emitter> BreakEffect; // Emitter class for the breaking effect

var() StaticMesh StaticMeshNormal; // Regular (non-broken) static mesh. If None, inherited from defaults.
var() StaticMesh StaticMeshBroken; // Broken static mesh. If None, simply hide.
var() array<Material> SkinsNormal; // What skins to use when not broken. If None, inherited from defaults.
var() array<Material> SkinsBroken; // Skins to use when broken. If None, don't change skins.

var() Sound BreakEffectSound; // What sound to play when I get broken
var() Sound RespawnEffectSound; // What sound to play when I respawn
var() Sound AmbientSoundNormal; // Ambient sound when not broken
var() Sound AmbientSoundBroken; // Ambient sound when broken

var() array< class<DamageType> > DamageClasses; // Array of DamageTypes that I either filter out or only accept
var() enum EDamageFilterType // How I treat incoming damage.
{
	DFT_AnyDamage, // Accept all incoming damage.
	DFT_FilterSpecified, // Accept all incoming damage EXCEPT those DamageTypes specified in DamageClasses.
	DFT_SpecifiedDamageOnly, // Accept ONLY the DamageTypes specified in DamageClasses.
	DFT_IgnoreAllDamage // Ignore all damage -- can only be exploded by a triggered event
} WhatHurtsMe;

replication
{
	reliable if (Role == ROLE_Authority && bNetInitial)
		StaticMeshNormal, StaticMeshBroken, SkinsNormal, SkinsBroken;

	reliable if (Role == ROLE_Authority)
		SetToNormal, SetToBroken;
}

// PostBeginPlay
// Call super, and set our normal static mesh/skins.
event PostBeginPlay()
{
	Super(Prop).PostBeginPlay();
	SetToNormal();
}

// SetToNormal
// Reset our max health and set our normal static mesh/skins.
simulated event SetToNormal()
{
	local int i;

	// Reset our max health, and "un-hide" us
	if (Role == ROLE_Authority)
	{
		Health = HealthMax;
		bHidden = False;
	}

	// Reset our static mesh. If no StaticMeshNormal was specified by the mapmaker, we set it to the default StaticMesh here.
	if (StaticMeshNormal == None && StaticMesh != StaticMeshBroken)
		StaticMeshNormal = StaticMesh;
	else
		SetStaticMesh(StaticMeshNormal);

	// Set ambient sounds
	if (AmbientSoundNormal == None && AmbientSound != AmbientSoundBroken)
		AmbientSoundNormal = AmbientSound;

	AmbientSound = AmbientSoundNormal;

	// Set skins (if specified)
	for (i=0; i<SkinsNormal.Length; i++)
		if (SkinsNormal[i] != None)
			Skins[i] = SkinsNormal[i];
}

// SetToBroken
// Zero our health, and either hide us or change to our broken mesh.
simulated event SetToBroken()
{
	local int i;

	if (Role == ROLE_Authority)
		Health = 0;

	// Switch to our broken static mesh. If we have no broken static mesh, we go hidden.
	if (StaticMeshBroken == None)
	{
		if (Role == ROLE_Authority)
			bHidden = True;
	}
	else
		SetStaticMesh(StaticMeshBroken);

	// Set our broken ambience
	if (AmbientSoundBroken != None)
		AmbientSound = AmbientSoundBroken;

	// Set skins (if specified)
	for (i=0; i<SkinsBroken.Length; i++)
		if (SkinsBroken[i] != None)
			Skins[i] = SkinsBroken[i];
}

// AcceptThisDamage
// Returns true if this incoming damage will affect us.
function bool AcceptThisDamage(class<DamageType> damageType)
{
	local int i;
	local bool bAccept;

	if (WhatHurtsMe == DFT_IgnoreAllDamage)
		return false;

	if (WhatHurtsMe == DFT_AnyDamage)
		return true;

	bAccept = false;
	for (i=0; i<DamageClasses.Length; i++)
	{
		if (WhatHurtsMe == DFT_FilterSpecified && ClassIsChildOf(DamageType, DamageClasses[i]))
			return false;

		if (WhatHurtsMe == DFT_SpecifiedDamageOnly && ClassIsChildOf(DamageType, DamageClasses[i]))
			bAccept = true;
	}

	return bAccept;
}

// BlowThisUp
// Cause our break explosion. Spawn breaking effects, set to dead, and go to Broken state.
singular function BlowThisUp(int Damage, vector HitLocation, vector Momentum)
{
	// set to dead
	SetToBroken();

	// Spawn effect so we don't have to record the hit values and
	// do it later in Broken beginstate or something. It's just
	// more efficient here
	Spawn(BreakEffect, Instigator,, Location, Rotation);

	if (BreakEffectSound != None)
		PlaySound(BreakEffectSound);

	// Say we're broken so we won't break anymore
	GotoState('Broken');
}

// TakeDamage
// If the incoming damage is strong enough, it deducts from our health. Explode if we hit zero health.
function TakeDamage(int Damage, Pawn instigatedBy, Vector hitlocation,
						Vector momentum, class<DamageType> damageType)
{
	// Check first if we take this damage
	if(!AcceptThisDamage(damageType))
		return;

	// Only remove health if the singular damage was strong enough
	if(damage > MinDamage)
	{
		Health -= damage;
		// if you run out of health, break
		if(Health <= 0)
			BlowThisUp(Damage, HitLocation, Momentum);
	}
}

// state Broken
// Wait to respawn, or destroy if we don't respawn.
state Broken
{
	ignores TakeDamage;

	// BeginState
	// Set our respawn timer or destroy ourself.
	function BeginState()
	{
		if (RespawnTime > 0)
			SetTimer(RespawnTime, false);
		else if (bHidden)
			Destroy();
	}

	// Timer
	// Reset ourself and exit the Broken state.
	event Timer()
	{
		SetToNormal();
		GotoState('');
	}
}

// defaultproperties
// Properties of our defaults!
defaultproperties
{
	RemoteRole=ROLE_SimulatedProxy
	DrawType=DT_StaticMesh
	StaticMesh=StaticMesh'Zo_Meshes.Karma.zo_barrel1'
	bCollideActors=True
	bBlockActors=True
	bBlockKarma=False
	bBlockPlayers=True
	bProjTarget=True
	bBlockZeroExtentTraces=True
	bBlockNonZeroExtentTraces=True
	bPathColliding=True

	MinDamage=4
	HealthMax=24
	RespawnTime=60
	RespawnEffect=None
	BreakEffect=class'STPExtras.PropExplosion'
	StaticMeshNormal=StaticMesh'Zo_Meshes.Karma.zo_barrel1'
	StaticMeshBroken=StaticMesh'Zo_Meshes.Karma.zo_barrel_exploded'
	BreakEffectSound=Sound'MiscSounds.Props.CarExplode'
	WhatHurtsMe=DFT_SpecifiedDamageOnly
	DamageClasses[0]=class'BaseFX.BulletDamage'
	DamageClasses[1]=class'BaseFX.BurnedDamage'
	DamageClasses[2]=class'BaseFX.ExplodedDamage'
}
